<?php
/**
 * @name Server
 * @description Classe de base gérant les serveurs du plugin "Server"
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 07/11/2012
 */
include_once SNMP_CLASS;
include_once LIB_SNMP;
include_once DEVICE_CLASS;
class SyslogServerException extends DeviceException{
}
class SyslogServer extends Device
{
	const OUTPUT_JSON = 1;
	const OUTPUT_OBJECT = 2;
	const OUTPUT_ARRAY = 3;
	protected $url;
	/*@NAME: __construct
	 *@IN: $dev un objet Device
	*@OUT: /
	*@TREAT: permet de construire un objet SyslogServer à partir d'un objet "Device"
	*@EXCEPTION: 	/
	*/
	public function __construct($dev=NULL)
	{
		if(get_class($dev) == 'Device')
		{
			parent::constructDevice($dev);
		}
		else
		{
			parent::__construct();
		}
	}
	public function setUrl($u)
	{
		$this->url = $u;
	}
	public function getUrl()
	{
		return $this->url;
	}
	private function formatOutput($data,$output)
	{
		if($output == SyslogServer::OUTPUT_JSON)
		{
			return $data;
		}
		elseif($output == SyslogServer::OUTPUT_ARRAY)
		{
			return json_decode($data,TRUE);
		}
		else
		{
			return json_decode($data);
		}
	}
	public function getLogByDay($output=SyslogServer::OUTPUT_OBJECT)
	{
		$stat = file_get_contents($this->url."/stat.php?action=byday");
		if($stat === FALSE)
		{
			$err = error_get_last();
			throw new SyslogServerException("Host not found: ".$err['message']);
		}
		return $this->formatOutput($stat,$output);
	}
	public function top10HostByHour(int $hour = 24,$output=SyslogServer::OUTPUT_OBJECT)
	{

		$stat = file_get_contents($this->url."/stat.php?action=tophost&hour=".$hour);
		if($stat === FALSE)
		{
			$err = error_get_last();
			throw new SyslogServerException("Host not found: ".$err['message']);
		}
		return $this->formatOutput($stat,$output);
	}
	public function severityByHour(int $hour = 24,$output=SyslogServer::OUTPUT_OBJECT)
	{

		$stat = file_get_contents($this->url."/stat.php?action=severity&hour=".$hour);
		if($stat === FALSE)
		{
			$err = error_get_last();
			throw new SyslogServerException("Host not found: ".$err['message']);
		}
		return $this->formatOutput($stat,$output);
	}
	public function getLogsByDay(int $day = 1,$output=SyslogServer::OUTPUT_OBJECT,$params=array())
	{
		$u = '';
		foreach($params as $key => $par)
		{
			$u .= '&'.$key.'='.urlencode($par);
		}
		//echo $this->url."/logs.php?day=".$day.$u;
		$stat = file_get_contents($this->url."/logs.php?day=".$day.$u);
		if($stat === FALSE)
		{
			$err = error_get_last();
			throw new SyslogServerException("Host not found: ".$err['message']);
		}
		return $this->formatOutput($stat,$output);
	}
	public function getConfiguration($output=SyslogServer::OUTPUT_OBJECT)
	{
		$conf = file_get_contents($this->url."/conf.php?action=cleanrule");
		if($conf === FALSE)
		{
			$err = error_get_last();
			throw new SyslogServerException("Host not found: ".$err['message']);
		}
		elseif($conf == "can't open rule file ")
		{
			throw new SyslogServerException("Bad permission to rule file on syslog");
		}
		return $this->formatOutput($conf,$output);
	}
	public function sendCurlConf($post)
	{
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $this->url."/conf.php");
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_SAFE_UPLOAD, FALSE);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
		$ret = curl_exec($ch);
		if($ret === FALSE)
		{
			$errno = curl_errno($ch);
			if($errno)
			{
				$error_message = curl_strerror($errno);
				throw new SyslogServerException("cURL error ({$errno}):\n {$error_message}");
			}
		}
		$httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		if($httpcode == 500)
		{
			throw new SyslogServerException("HTTP ERROR ON ".$this->url." $ret");
		}
		curl_close($ch);
		return $ret;
	}
	public function setConfiguration($array)
	{
		$array['action'] = 'cleanrule';
		$this->sendCurlConf($array);
	}
	public function getRules($id=NULL,$output=SyslogServer::OUTPUT_OBJECT)
	{
		$conf = FALSE;
		if(!empty($id))
		{
			if(is_numeric($id))
			{
				$conf = file_get_contents($this->url."/conf.php?action=rules&id=".$id);
			}
			else
			{
				 throw new SyslogServerException("Bad parameter id is not numeric");
			}
		}
		else
		{
			$conf = file_get_contents($this->url."/conf.php?action=rules");
		}
		if($conf === FALSE)
		{
			$err = error_get_last();
			throw new SyslogServerException("Host not found: ".$err['message']);
		}
		elseif($conf == "can't open rule file ")
		{
			throw new SyslogServerException("Bad permission to rule file on syslog");
		}
		return $this->formatOutput($conf,$output);
	}
	public function setOrderRules($ruleslist,$output=SyslogServer::OUTPUT_OBJECT)
	{
		$array['action'] = 'orderRules';
		$array['order'] = $ruleslist;
		//print_r($ruleslist);
		$ret = $this->sendCurlConf($array);
		return $this->formatOutput($ret,$output);
	}
	public function addRules($array,$output=SyslogServer::OUTPUT_OBJECT)
	{
		$array['action'] = 'rules';
	   // print_r($array);
		$ret = $this->sendCurlConf($array);
		return $this->formatOutput($ret,$output);
	}
	public function deleteRules($id,$output=SyslogServer::OUTPUT_OBJECT)
	{
		$array['action'] = 'deleterules';
		$array['id'] = json_encode(array($id));
		$ret = $this->sendCurlConf($array);
		return $this->formatOutput($ret,$output);
	}
	public function getHosts($output=SyslogServer::OUTPUT_OBJECT)
	{
		$conf = FALSE;
		$conf = file_get_contents($this->url."/conf.php?action=getHosts");
		if($conf === FALSE)
		{
			$err = error_get_last();
			throw new SyslogServerException("Host not found: ".$err['message']);
		}
		elseif($conf == "can't open rule file ")
		{
			throw new SyslogServerException("Bad permission to rule file on syslog");
		}
		return $this->formatOutput($conf,$output);
	}
}
